using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Security.Permissions;
using System.Windows.Forms;
using ivware.Mobile.RAPISync.Core;
using ivware.Mobile.RAPISync.GUI.Controls;
using ivware.Mobile.RAPISync.Store;
using OpenNETCF.Desktop.Communication;

namespace ivware.Mobile.RAPISync.GUI
{
	public partial class MainGUI : Form
	{
		#region Data Members

		private BatchProfile m_CurrentBatch = new BatchProfile();
		private ProfileList m_SelectedProfiles = new ProfileList();
		private Profile m_CurrentProfile;
		private Timer m_UITimer = new Timer();
		private const string NewPairName = "New Folder Pair";
		private const int UI_REFRESH_TIMEOUT = 400;
		private const int UI_SPLASH_TIMEOUT = 1000;
		private uint m_ActivateMessage = SingleInstance.ActivateMessage;

		public delegate bool StoreItemEventDelegate(IFileStore i_store, StoreItem i_source, StoreItem i_destination);

		public delegate void ProfileEventDelegate(Profile i_source);

		#endregion

		#region Constructors

		public MainGUI()
		{
			InitializeComponent();

			FormClosing += new FormClosingEventHandler(OnFormClosing);
			m_UITimer.Tick += new EventHandler(UIUpdateEvent);
			TrayIcon.Icon = Resources.pda;
		}

		#endregion

		#region Profile Event Notifications

		private void SubscribeProfile(Profile i_prf)
		{
			i_prf.BeforeCopy += new OnBeforeCopy(OnBeforeCopy);
			i_prf.BeforeDelete += new OnBeforeDelete(OnBeforeDelete);
			i_prf.BeforeExecute += new OnBeforeExecuteProfile(OnBeforeExecute);
			i_prf.AfterExecute += new OnAfterExecuteProfile(OnAfterExecute);
			i_prf.OnError += new OnProfileError(OnError);
			i_prf.AfterCopy += new OnAfterCopy(OnAfterCopy);
		}

		private void OnAfterCopy(IFileStore i_store, StoreItem i_source, StoreItem i_destination, Exception ex)
		{
			if (ex == null)
				OutputLog.WriteLine(string.Format("   Done!"));
			else
				OutputLog.WriteLine(string.Format("   Failed! - " + ex.Message));
		}

		private bool OnBeforeDelete(IFileStore i_source, StoreItem i_destination)
		{
			OutputLog.WriteLine(string.Format("   Deleting {0}...", i_destination.ItemRelativePath));
			return true;
		}

		private bool OnBeforeCopy(IFileStore i_store, StoreItem i_source, StoreItem i_destination)
		{
			if (InvokeRequired)
			{
				StoreItemEventDelegate cbDel =
					new StoreItemEventDelegate(OnBeforeCopy);

				return (bool) Invoke(cbDel, i_store, i_source, i_destination);
			}

			if (i_source.IsDirectory)
			{
				OutputLog.Write(string.Format("   Creating Folder {0}...", i_destination.ItemRelativePath));
			}
			else
			{
				OutputLog.Write(string.Format("   Copying {0} - {1}...", i_source.ItemRelativePath, i_source.SizeAsString));
				string path = Path.GetFileName(i_source.ItemPath);
				if (path.Length > 40)
					path = path.Substring(0, 30) + "...";
				TrayIcon.Text = string.Format("Copying {0} - {1}..", path, i_source.SizeAsString);
			}

			return true;
		}

		private void OnAfterExecute(Profile i_source)
		{
			if (InvokeRequired)
			{
				ProfileEventDelegate cbDel =
					new ProfileEventDelegate(OnAfterExecute);

				Invoke(cbDel, i_source);
				return;
			}

			if (i_source.Diffgram.IsValid)
			{
				if (i_source.HasFailedItems)
				{
					OutputLog.WriteLine("\n   There were errors during synchronization!");

					ItemDiffGramList failed = i_source.Diffgram.GetByResult(OperationResult.ResultStatus.Failed);

					foreach (ItemDiffGram i in failed)
					{
						OutputLog.WriteLine("   Source: " + i.Source.ItemPath);
						OutputLog.WriteLine("   Destination: " + i.Destination.ItemPath);
						OutputLog.WriteLine("   Status: " + i.Result);
						OutputLog.WriteLine("");
					}
				}

				if (i_source.MadeChanges)
				{
					OutputLog.WriteLine(string.Format("   Copied {0}, Deleted {1}, Unchanged: {2}",
					                                  i_source.Diffgram.GetByActionAndResult(ItemDiffGram.SyncAction.CopyToDestination,
					                                                                         OperationResult.ResultStatus.Success).
					                                  	Count,
					                                  i_source.Diffgram.GetByActionAndResult(
					                                  	ItemDiffGram.SyncAction.DeleteFromDestination,
					                                  	OperationResult.ResultStatus.Success)
					                                  	.Count,
					                                  i_source.Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count));

					if (i_source.HasFailedItems)
						TrayIcon.ShowBalloonTip(1000, i_source.Name + " Completed with Errors!",
						                        "Some files were not syncronized.\nPlease check the syncronization\nlog for details!",
						                        ToolTipIcon.Error);
					else
						TrayIcon.ShowBalloonTip(1000, i_source.Name + " Completed!",
						                        string.Format("Copied: {0}, Deleted: {1} files\nCompleted in {2:F0} min.",
						                                      i_source.Diffgram.GetByActionAndResult(
						                                      	ItemDiffGram.SyncAction.CopyToDestination,
						                                      	OperationResult.ResultStatus.Success).Count,
						                                      i_source.Diffgram.GetByActionAndResult(
						                                      	ItemDiffGram.SyncAction.DeleteFromDestination,
						                                      	OperationResult.ResultStatus.Success)
						                                      	.Count,
						                                      i_source.RunningTime.TotalMinutes), ToolTipIcon.Info);
				}
				else
				{
					OutputLog.WriteLine("   Nothing to do!");
				}

				OutputLog.WriteLine(
					string.Format("Synchronization completed in {0:F0} min.", i_source.RunningTime.TotalMinutes));
			}
			else
			{
				OutputLog.WriteLine("Synchronization failed!");
			}

			UpdateProfileStatus(i_source);
		}

		private bool OnBeforeExecute(Profile i_source)
		{
			OutputLog.WriteLine(
				string.Format("\nStarting profile: {0} {1} {2}...",
				              i_source.Name,
				              i_source.Rule.IsTestRun == true ? "(test run)" : "",
				              i_source.Rule.CopyOnceOnly == true ? "(copy once)" : ""));

			UpdateProfileStatus(i_source);
			return true;
		}

		private void OnError(Profile i_source)
		{
			if (InvokeRequired)
			{
				ProfileEventDelegate cbDel =
					new ProfileEventDelegate(OnError);

				Invoke(cbDel, i_source);
			}
			else
			{
				OutputLog.WriteLine("Error occured while synchronizing " + i_source.Name + "\nError: " +
				                    i_source.LastException.Message);
				TrayIcon.ShowBalloonTip(2000, "Failed to sync " + i_source.Name, i_source.LastException.Message, ToolTipIcon.Error);
			}
		}

		#endregion

		#region UI Event Handlers

		private void OnFormClosing(object sender, FormClosingEventArgs e)
		{
			if (m_CurrentBatch.IsRunning)
				m_CurrentBatch.CancelBatch();

			Properties.Settings.Default.Save();
			m_CurrentBatch.SaveToFile(Configuration.DefaultProfilePath);
		}

		private void UIUpdateEvent(object sender, EventArgs e)
		{
			if (m_UITimer.Interval == UI_SPLASH_TIMEOUT && m_CurrentBatch.Profiles.Count == 0)
			{
				m_UITimer.Enabled = false;
				Welcome w = new Welcome();
				w.ShowDialog();
				m_UITimer.Interval = UI_REFRESH_TIMEOUT;
				m_UITimer.Enabled = true;
			}

			if (m_SelectedProfiles.Count == 0)
			{
				ExecuteSelected.Enabled = false;
				DeletePair.Enabled = false;
			}
			else
			{
				ExecuteSelected.Enabled = true;
				DeletePair.Enabled = true;
			}

			if (m_CurrentBatch.Profiles.Count == 0)
			{
				ExecuteAll.Enabled = false;
			}
			else
			{
				ExecuteAll.Enabled = true;
			}

			bool enable_target_selection;

			if (m_CurrentProfile != null)
			{
				btnAdd.Visible = true;
				EnableDisableEntryControls(true);
				enable_target_selection = true;
			}
			else
			{
				btnAdd.Visible = false;
				ClearProfileEntry();
				EnableDisableEntryControls(false);
				enable_target_selection = false;
			}

			if (m_CurrentBatch.IsRunning)
			{
				if (TrayIcon.Text == "Waiting to Start")
					TrayIcon.Text = "Running...";

				CancelRun.Visible = true;
				mnuExecuteAll.Text = "Stop";
			}
			else
			{
				if (RAPIUtils.DeviceConnected)
				{
					SelectTargetMobile.Text = "Browse for Device Folder...";
					TrayIcon.Text = "Waiting to Start";
				}
				else
				{
					SelectTargetMobile.Text = "Mobile Device not connected!";
					enable_target_selection = false;
					TrayIcon.Text = "Mobile Device is Not Connected";
				}

				CancelRun.Visible = false;
				mnuExecuteAll.Text = "Execute All";
			}

			SelectTargetMobile.Enabled = enable_target_selection;
		}

		private void OnLoad(object sender, EventArgs e)
		{
			try
			{
				m_CurrentBatch = BatchProfile.LoadFromFile(Configuration.DefaultProfilePath);
			}
			catch (FileNotFoundException)
			{
				OutputLog.WriteLine("Unable to find the default profile. New profile will be created!");
			}
			catch (Exception ex)
			{
				MessageBox.Show("Unable to load the default ptofile!\nReason:" + ex.Message);
			}

			//Subscribe to each profile in the batch
			foreach (Profile prf in m_CurrentBatch.Profiles)
			{
				SubscribeProfile(prf);
			}

			ExecuteOnStart.Checked = Properties.Settings.Default.AutoexecuteOnStart;
			StartMinimized.Checked = Properties.Settings.Default.StartMinimized;

			if (StartMinimized.Checked)
				WindowState = FormWindowState.Minimized;
			OnResize(null, null);

			if (!RAPIUtils.ActiveSyncPresent)
			{
				DestIsPCFolder.Checked = true;
				DestIsMobileDevice.Enabled = false;
				AutoStart.Visible = false;
			}
			else
			{
				AutoStart.Checked = IsAutostartOnConnect();
				RAPIUtils.Instance.RAPI.RAPIConnected += new RAPIConnectedHandler(OnDeviceConnectedDisconnected);
				RAPIUtils.Instance.RAPI.RAPIDisconnected += new RAPIConnectedHandler(OnDeviceConnectedDisconnected);
			}

			LoadProfileList();

			if (Properties.Settings.Default.AutoexecuteOnStart)
				OnExecuteAllClicked(null, null);

			if (m_CurrentBatch.Profiles.Count == 0)
				m_UITimer.Interval = UI_SPLASH_TIMEOUT;
			else
				m_UITimer.Interval = UI_REFRESH_TIMEOUT;

			if (!RAPIUtils.ActiveSyncPresent)
			{
				OutputLog.WriteLine(
					"Microsoft ActiveSync is not installed. You will not be able to synchronize to a Windows Mobile device!");
			}

			m_UITimer.Enabled = true;
		}

		private void OnDeviceConnectedDisconnected()
		{
				OutputLog.WriteLine(RAPIUtils.DeviceConnected ? "Mobile Device Connected" : "Mobile Device Disconnected");

			if (RAPIUtils.DeviceConnected && AutoStart.Checked)
			{
				OnExecuteAllClicked(null, null);
			}
		}

		private void OnSelectSourceClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			DialogResult res = SourceBrowser.ShowDialog();
			if (res == DialogResult.OK)
			{
				SourcePath.Text = SourceBrowser.SelectedPath;

				if (FolderPairName.Text == NewPairName)
				{
					FolderPairName.Text = Path.GetFileName(SourceBrowser.SelectedPath);
				}
			}
		}

		private void OnExecuteAllClicked(object sender, EventArgs e)
		{
			ExecuteProfiles(false);
		}

		private void OnProfileSelected(object sender, EventArgs e)
		{
			m_SelectedProfiles.Clear();

			ListView.SelectedListViewItemCollection selected = ProfileList.SelectedItems;

			foreach (ListViewItem i in selected)
			{
				m_SelectedProfiles.Add(i.Tag as Profile);
			}

			if (m_SelectedProfiles.Count > 0)
			{
				m_CurrentProfile = m_SelectedProfiles[0];
				EditProfile();
			}
			else
			{
				m_CurrentProfile = null;
			}

			Text = "BeyondSync" + " " + Configuration.ProductVersion;
		}

		private void OnAddNewPairClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			m_CurrentProfile = new Profile();
			m_CurrentProfile.Name = NewPairName;
			m_CurrentProfile.CopyOnce = true;
			m_CurrentProfile.Rule = new SyncRule(SyncRule.RuleType.SourceContributesToDestination);
			EditProfile();
			SourcePath.Focus();
		}

		private void OnAddClicked(object sender, EventArgs e)
		{
			ApplyChanges();
		}

		private void OnDeletePairClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			if (
				MessageBox.Show("Folder Pair " + m_CurrentProfile.Name + " will be deleted!\nAre you sure?", "Confirm Delete",
				                MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2,
				                MessageBoxOptions.DefaultDesktopOnly, false) == DialogResult.No)
				return;

			if (m_CurrentProfile != null)
			{
				m_CurrentBatch.Profiles.Remove(m_CurrentProfile);
				m_CurrentProfile.CleanHistory();
			}

			LoadProfileList();
		}

		private void OnSelectTargetClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			DeviceFolderSelect dlg = new DeviceFolderSelect();
			if (dlg.ShowDialog() == DialogResult.OK)
			{
				if (dlg.SelectedFolder != null)
					DestinationPath.Text = dlg.SelectedFolder;
			}
		}

		private void OnSelectTargetPCClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			DialogResult res = SourceBrowser.ShowDialog();
			if (res == DialogResult.OK)
			{
				DestinationPath.Text = SourceBrowser.SelectedPath;
			}
		}

		private void OnExecuteSelectedClicked(object sender, EventArgs e)
		{
			ExecuteProfiles(true);
		}

		private void OnAutoStartCheckedChanged(object sender, EventArgs e)
		{
			if (AutoStart.Checked)
			{
				if (!IsAutostartOnConnect())
					RAPIUtils.Instance.RAPI.ActiveSync.AutoStartOnConnect.Add(Assembly.GetExecutingAssembly().Location);
			}
			else
			{
				RAPIUtils.Instance.RAPI.ActiveSync.AutoStartOnConnect.Remove(Assembly.GetExecutingAssembly().Location);
			}
		}

		private void OnExecuteOnStartCheckedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.AutoexecuteOnStart = ExecuteOnStart.Checked;
		}

		private void OnCancelRunClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			CancelRun.Text = "Canceling...";
			m_CurrentBatch.CancelBatch();
		}

		private void OnResize(object sender, EventArgs e)
		{
			if (WindowState == FormWindowState.Minimized)
				Hide();

			TrayIcon.Visible = (WindowState == FormWindowState.Minimized);
			ShowInTaskbar = !TrayIcon.Visible;
		}

		private void OnTrayDoubleClick(object sender, EventArgs e)
		{
			OnMenuOpen(null, null);
		}

		private void OnMenuOpen(object sender, EventArgs e)
		{
			Show();
			WindowState = FormWindowState.Normal;
		}

		private void OnMenuExecuteAll(object sender, EventArgs e)
		{
			if (m_CurrentBatch.IsRunning)
			{
				OnCancelRunClicked(null, null);
			}
			else
				OnExecuteAllClicked(null, null);
		}

		private void OnMenuExit(object sender, EventArgs e)
		{
			Close();
		}

		private void OnStartMinimizedChanged(object sender, EventArgs e)
		{
			Properties.Settings.Default.StartMinimized = StartMinimized.Checked;
		}

		private void OnReportProblemClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			try
			{
				Process.Start("mailto:beyondpod@gmail.com&subject=BeyondSync " + Configuration.ProductVersion +
				              " problem report&Body=Please enter the problem description here...");
			}
			catch
			{
				MessageBox.Show("Unable to start youe e-mail client!", "Sorry");
			}
		}

		private void OnDestIsMobileDeviceChanged(object sender, EventArgs e)
		{
			if (DestIsMobileDevice.Checked)
			{
				SelectTargetPC.Visible = false;
				SelectTargetMobile.Visible = true;
			}
			else
			{
				SelectTargetPC.Visible = true;
				SelectTargetMobile.Visible = false;
			}
		}

		private void OnFolderPairDoubleClick(object sender, EventArgs e)
		{
			ExecuteProfiles(true);
		}

		[PermissionSet(SecurityAction.Demand, Name = "FullTrust")]
		protected override void WndProc(ref Message msg)
		{
			if (msg.Msg == m_ActivateMessage)
			{
				if (WindowState == FormWindowState.Minimized)
				{
					TrayIcon.ShowBalloonTip(1000, "BeyondSync", "BeyondSync is already running!", ToolTipIcon.Info);
				}
			}

			// call the base class WndProc for default message handling
			//
			base.WndProc(ref msg);
		}

		#endregion

		#region Helper Methods

		private void UpdateProfileStatus(Profile i_Profile)
		{
			if (InvokeRequired)
			{
				ProfileEventDelegate cbDel =
					new ProfileEventDelegate(UpdateProfileStatus);

				Invoke(cbDel, i_Profile);
			}
			else
			{
				foreach (ListViewItem lvi in ProfileList.Items)
				{
					if (lvi.Tag == i_Profile)
					{
						lvi.SubItems[1].Text = i_Profile.ProfileState.ToString().Replace("_", " ");

						if (i_Profile.ProfileState != Profile.State.Running)
						{
							if (i_Profile.Diffgram.IsValid)
							{
								if (i_Profile.MadeChanges)
									lvi.SubItems[1].Text += string.Format(" - Copied: {0}, Deleted: {1}", i_Profile.Diffgram.GetByActionAndResult(
									                                                                      	ItemDiffGram.SyncAction.
									                                                                      		CopyToDestination,
									                                                                      	OperationResult.ResultStatus.Success).
									                                                                      	Count,
									                                      i_Profile.Diffgram.GetByActionAndResult(
									                                      	ItemDiffGram.SyncAction.DeleteFromDestination,
									                                      	OperationResult.ResultStatus.Success)
									                                      	.Count);
								else
									lvi.SubItems[1].Text += " - Nothing Changed";
							}
							else
							{
								lvi.SubItems[1].Text += " - " + i_Profile.Diffgram.LastError.Message;
							}
						}

						switch (i_Profile.ProfileState)
						{
							case Profile.State.Canceled:
								lvi.SubItems[1].ForeColor = Color.Orange;
								break;

							case Profile.State.Failed:
								lvi.SubItems[1].ForeColor = Color.Red;
								break;

							case Profile.State.Completed:
								if (i_Profile.MadeChanges)
									lvi.SubItems[1].ForeColor = Color.Green;
								else
									lvi.SubItems[1].ForeColor = Color.Gray;
								break;

							case Profile.State.Running:
								lvi.SubItems[1].ForeColor = Color.Navy;
								break;
						}
					}
				}
			}
		}

		private void ClearProfileEntry()
		{
			SourcePath.Text = String.Empty;
			DestinationPath.Text = String.Empty;
			FolderPairName.Text = String.Empty;
			CopyOnce.Checked = false;
			AllowDestinationDeletions.Checked = false;
		}

		private void EnableDisableEntryControls(bool i_enable)
		{
			SourcePath.Enabled = i_enable;
			DestinationPath.Enabled = i_enable;
			FolderPairName.Enabled = i_enable;
			CopyOnce.Enabled = i_enable;
			SelectSource.Enabled = i_enable;
			DestIsPCFolder.Enabled = i_enable;
			AllowDestinationDeletions.Enabled = i_enable;

			if (RAPIUtils.ActiveSyncPresent)
				DestIsMobileDevice.Enabled = i_enable;
		}

		private void LoadProfileList()
		{
			ProfileList.Items.Clear();

			foreach (Profile prf in m_CurrentBatch.Profiles)
			{
				string[] items = new string[2];

				items[0] = prf.Name;
				items[1] = prf.ProfileState.ToString().Replace("_", " ");

				ListViewItem lvi = new ListViewItem(items);
				lvi.Tag = prf;
				lvi.UseItemStyleForSubItems = false;
				ProfileList.Items.Add(lvi);

				if (prf.Destination.GetStore() is CEStore)
					lvi.ImageIndex = 0;
				else
					lvi.ImageIndex = 1;
			}

			OnProfileSelected(null, null);
		}

		private void ApplyChanges()
		{
			string s = SourcePath.Text.Trim();
			string d = DestinationPath.Text.Trim();
			string n = FolderPairName.Text.Trim();

			if (s.Length == 0 || d.Length == 0 || n.Length == 0)
			{
				MessageBox.Show("Please enter the Source path, the Destination and the Name of the profile!", "Error",
				                MessageBoxButtons.OK, MessageBoxIcon.Error,
				                MessageBoxDefaultButton.Button1);
				return;
			}

			if (string.Compare(s, d, true) == 0)
			{
				MessageBox.Show("Both Source and Destination point to the same location!", "Error", MessageBoxButtons.OK,
				                MessageBoxIcon.Error,
				                MessageBoxDefaultButton.Button1);
				return;
			}

			if (m_CurrentProfile == null)
			{
				m_CurrentProfile = new Profile();
			}

			//Check if the profile with the same name already exists
			Profile existing = m_CurrentBatch.GetProfileByName(n);
			if (existing != null && existing != m_CurrentProfile)
			{
				MessageBox.Show("Profile with this name already exist!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error,
				                MessageBoxDefaultButton.Button1);
				FolderPairName.Focus();
				return;
			}

			m_CurrentProfile.Source = new SyncEndPoint(s);

			//If the store is a mobile device
			if (DestIsMobileDevice.Checked)
				d = CEStore.STORE_CEFILE + d;

			m_CurrentProfile.Destination = new SyncEndPoint(d);

			if (AllowDestinationDeletions.Checked)
				m_CurrentProfile.Rule = new SyncRule(SyncRule.RuleType.SourceOverridesDestination);
			else
				m_CurrentProfile.Rule = new SyncRule(SyncRule.RuleType.SourceContributesToDestination);

			m_CurrentProfile.CopyOnce = CopyOnce.Checked;
			m_CurrentProfile.Name = n;

			if (!m_CurrentBatch.Profiles.Contains(m_CurrentProfile))
			{
				m_CurrentBatch.Profiles.Add(m_CurrentProfile);
				SubscribeProfile(m_CurrentProfile);
			}

			LoadProfileList();

			m_CurrentProfile = null;
		}

		private void EditProfile()
		{
			if (m_CurrentProfile.Source != null)
				SourcePath.Text = m_CurrentProfile.Source.Root.ItemPath;
			else
				SourcePath.Text = String.Empty;

			if (m_CurrentProfile.Destination != null)
				DestinationPath.Text = m_CurrentProfile.Destination.Root.ItemPath;
			else
				DestinationPath.Text = String.Empty;

			FolderPairName.Text = m_CurrentProfile.Name;
			CopyOnce.Checked = m_CurrentProfile.CopyOnce;


			if (m_CurrentProfile.Rule.Rule == SyncRule.RuleType.SourceOverridesDestination)
				AllowDestinationDeletions.Checked = true;
			else
				AllowDestinationDeletions.Checked = false;

			if (m_CurrentBatch.Profiles.Contains(m_CurrentProfile))
			{
				if (m_CurrentProfile.Destination.GetStore() is FileSystemStore)
					DestIsPCFolder.Checked = true;
				else
					DestIsMobileDevice.Checked = true;

				btnAdd.Text = "&Apply";
			}
			else
			{
				btnAdd.Text = "&OK";
			}

			EnableDisableEntryControls(true);
		}

		private void ExecuteProfiles(bool i_selected)
		{
			if (m_CurrentBatch.IsRunning)
			{
				return;
			}

			CancelRun.Text = "Cancel Run!";
			OutputLog.Clear();

			if (i_selected)
				m_CurrentBatch.ExecuteAllProfilesAsync(m_SelectedProfiles);
			else
				m_CurrentBatch.ExecuteAllProfilesAsync();
		}

		#endregion

		#region Properties

		private bool IsAutostartOnConnect()
		{
			Assembly.GetExecutingAssembly();

			foreach (string app in RAPIUtils.Instance.RAPI.ActiveSync.AutoStartOnConnect)
			{
				if (app.IndexOf(Assembly.GetExecutingAssembly().Location) >= 0)
					return true;
			}

			return false;
		}

		#endregion


	}
}