using System;
using System.IO;
using ivware.Mobile.RAPISync.Core;
using OpenNETCF.Desktop.Communication;

namespace ivware.Mobile.RAPISync.Store
{
	/// <summary>
	/// Summary description for CEStore.
	/// </summary>
	public class CEStore : IFileStore
	{
		private StoreItem m_Root;
		private StoreItemList m_ItemList;
		public const string STORE_CEFILE = "cefile:";
		public const int MAX_CE_PATH = 128;
		private bool m_Initialized = false;

		/// <summary>
		/// Fired before Copy
		/// </summary>
		public event OnBeforeCopy BeforeCopy;

		/// <summary>
		/// Fired after copy
		/// </summary>
		public event OnAfterCopy AfterCopy;

		/// <summary>
		/// Fired before Delete
		/// </summary>
		public event OnBeforeDelete BeforeDelete;

		/// <summary>
		/// Fired after delete
		/// </summary>
		public event OnAfterDelete AfterDelete;

		/// <summary>
		/// Description of the store
		/// </summary>
		public string Description
		{
			get { return "Window CE/MOBILE/SMARTPHONE File system"; }
		}

		/// <summary>
		/// Does not support free space information
		/// </summary>
		public bool CanGetFreeSpace
		{
			get { return false; }
		}

		public long FreeSpace
		{
			get { throw new NotImplementedException(); }
		}

		public StoreItemList Items
		{
			get
			{
				if (m_ItemList == null)
				{
					m_ItemList = GetFolderStructure(m_Root, RAPIUtils.Instance.RAPI);
					m_ItemList.Add(m_Root);
				}
				return m_ItemList;
			}
		}

		private StoreItemList GetFolderStructure(StoreItem i_Item, RAPI i_Handle)
		{
			StoreItemList list = new StoreItemList();


			if (i_Handle.DeviceFileExists(m_Root.ItemPath))
			{
				// Process the directory list
				FileList files = i_Handle.EnumFiles(Path.Combine(i_Item.ItemPath, "*.*"));
				if (files != null)
				{
					foreach (FileInformation entry in files)
					{
						CEStoreItem i = new CEStoreItem(entry, i_Item.ItemPath, this);
						list.Add(i);

						if (i.IsDirectory)
							list.AddRange(GetFolderStructure(i, i_Handle));
					}
				}
			}

			return list;
		}

		public void CopyItem(StoreItem i_source, StoreItem i_destination)
		{
			Exception copyProblem = null;

			if (i_source.IsDirectory != i_destination.IsDirectory)
				throw new ArgumentException(
					String.Format("Attempting to copy file into directory or vice versa!\n{0} -> {1}", i_source, i_destination));

			if (i_destination.ItemPath.Length > MAX_CE_PATH)
				throw new ArgumentException(
					String.Format("Destination path is too long. Maximum allowed path is 128 symbols. This path is {0} symbols", i_destination.ItemPath.Length));

			if (BeforeCopy != null && BeforeCopy(this, i_source, i_destination) != true)
				return;

			try
			{
				//Make sure that the path to 
				if (i_destination.IsPlaceholder)
					MakePathTo(i_destination, RAPIUtils.Instance.RAPI);

				if (!i_destination.IsDirectory)
					RAPIUtils.Instance.RAPI.CopyFileToDevice(i_source.ItemPath, i_destination.ItemPath, true);
			}
			catch (OutOfRemoteStorageSpaceException e)
			{
				OutOfStorageSpaceException ex = new OutOfStorageSpaceException(e.Message);
				ex.AvaialbleSpaceLeft = e.AvaialbleSpaceLeft;
				copyProblem = ex;
			}
			catch (Exception ex)
			{
				copyProblem = ex;
			}
			finally
			{
				if (AfterCopy != null)
					AfterCopy(this, i_source, i_destination, copyProblem);
			}

			if (copyProblem != null)
				throw copyProblem;
		}

		private void MakePathTo(StoreItem i_destination, RAPI i_handle)
		{
			//Get the path part
			string path = i_destination.ItemPath;

			if (!i_destination.IsDirectory)
				path = Path.GetDirectoryName(path);

			//Now make sure we have the entire path created on the device
			string[] parts = path.Split(StoreItem.DIR_SEPS);

			path = String.Empty + Path.DirectorySeparatorChar;
			int i = 0;
			do
			{
				path = Path.Combine(path, parts[i]);

				if (!i_handle.DeviceFileExists(path))
					i_handle.CreateDeviceDirectory(path);
			} while (++i != parts.Length);
		}

		public void DeleteItem(StoreItem i_destination)
		{
			Exception deleteException = null;

			try
			{
				//Check if the item was already deleted
				if (!RAPIUtils.Instance.RAPI.DeviceFileExists(i_destination.ItemPath))
					return;

				if (BeforeDelete != null && BeforeDelete(this, i_destination) != true)
					return;

				if (i_destination.IsDirectory)
				{
					RAPIUtils.Instance.RAPI.RemoveDeviceDirectory(i_destination.ItemPath, true);
				}
				else
				{
					RAPIUtils.Instance.RAPI.DeleteDeviceFile(i_destination.ItemPath);
				}
			}
			catch (Exception ex)
			{
				deleteException = ex;
			}
			finally
			{
				if (AfterDelete != null)
					AfterDelete(this, i_destination, deleteException);
			}

			if (deleteException != null)
				throw deleteException;
		}

		internal string CalculateRelativePath(StoreItem i_Item)
		{
			string root = m_Root.ItemPath.ToLower().Replace(@"\", "/");
			string item = i_Item.ItemPath.ToLower().Replace(@"\", "/");

			if (item.IndexOf(root) != 0)
				return string.Empty;

			string path = i_Item.ItemPath.Substring(root.Length);
			return path.Trim(StoreItem.DIR_SEPS);
		}

		public void InitializeRoot(StoreItem i_Root)
		{
			if (RAPIUtils.DeviceConnected)
			{
				CEStoreItem root = new CEStoreItem(i_Root.ItemPath, this);
				Refresh();
				m_Root = root;
				m_Initialized = true;
			}
			else
				m_Initialized = false;
		}

		/// <summary>
		/// Gets teh Root Item of the Store
		/// </summary>
		public StoreItem StoreRoot
		{
			get { return m_Root; }
		}

		/// <summary>
		/// Refreshes teh store
		/// </summary>
		public void Refresh()
		{
			m_ItemList = null;
		}

		public bool Initialized
		{
			get { return m_Initialized; }
		}

		public string Protocol
		{
			get { return STORE_CEFILE; }
		}
	}
}