using System;
using System.IO;
using System.Text;
using System.Threading;
using System.Xml;

namespace ivware.Mobile.RAPISync.Core
{
	public class BatchProfile
	{
		private ProfileList m_Profiles = new ProfileList();
		private ProfileList m_ProfilesToExecute = new ProfileList();
		private Thread m_AsyncExecutionThread;

		private delegate void ExecuteProfileDelegate(Profile i_Profile);

		public BatchProfile()
		{
		}

		/// <summary>
		/// Executes a batch profile made of mutiple command lines
		/// </summary>
		/// <param name="i_Profile"></param>
		/// <returns></returns>
		public BatchProfile(string i_Profile)
		{
			char[] dels = {'\n', '\r'};

			string[] profiles = i_Profile.Split(dels);

			foreach (string profile in profiles)
			{
				string profCmdLine = profile.Trim();
				if (profCmdLine.Length > 6 && !profCmdLine.StartsWith("rem ", StringComparison.InvariantCultureIgnoreCase))
				{
					Profile prf = LoadProfile(profCmdLine);
					m_Profiles.Add(prf);
				}
			}
		}

		/// <summary>
		/// Loads a single command line profile
		/// </summary>
		/// <param name="i_CmdLine"></param>
		/// <returns></returns>
		public static Profile LoadProfile(string i_CmdLine)
		{
			char[] dels = {','};
			string[] args = i_CmdLine.Split(dels);
			return LoadProfile(args);
		}

		/// <summary>
		/// Loads a profile wher eeach argument is on a different possition in a string array
		/// </summary>
		/// <param name="i_Args"></param>
		/// <returns></returns>
		public static Profile LoadProfile(string[] i_Args)
		{
			if (i_Args.Length < 3)
			{
				throw new ArgumentException("Not enough Arguments specified!");
			}

			Profile prf = new Profile();

			prf.Source = new SyncEndPoint(i_Args[0]);
			prf.Destination = new SyncEndPoint(i_Args[1]);

			if (i_Args[2].Trim() == "-o")
				prf.Rule = new SyncRule(SyncRule.RuleType.SourceOverridesDestination);
			else
				prf.Rule = new SyncRule(SyncRule.RuleType.SyncBothWays);

			if (i_Args.Length > 3)
				prf.Name = i_Args[3].Trim();

			if (i_Args.Length == 5 && i_Args[4].Trim() == "-h")
				prf.CopyOnce = true;

			if (i_Args.Length == 6 && i_Args[5].Trim() == "-t")
				prf.Rule.IsTestRun = true;

			return prf;
		}

		public static BatchProfile LoadFromFile(string i_path)
		{
			if (!File.Exists(i_path))
				throw new FileNotFoundException("FIle " + i_path + " does not exist!");

			BatchProfile bp = new BatchProfile();

			XmlTextReader tr = null;

			try
			{
				tr = new XmlTextReader(i_path);

				object syncPoint = tr.NameTable.Add("syncPoint");

				while (tr.Read())
				{
					if (tr.NodeType == XmlNodeType.Element && ReferenceEquals(tr.LocalName, syncPoint))
					{
						string sourcePath = String.Empty, destinationPath = String.Empty, name = String.Empty, id = string.Empty;
						int rule = (int) SyncRule.RuleType.SourceOverridesDestination;
						bool copyOnce = false;

						while (tr.MoveToNextAttribute())
						{
							switch (tr.LocalName)
							{
								case "sourcePath":
									sourcePath = tr.Value;
									break;

								case "destinationPath":
									destinationPath = tr.Value;
									break;

								case "rule":
									rule = tr.ReadContentAsInt();
									break;

								case "name":
									name = tr.Value;
									break;

								case "copyOnce":
									copyOnce = tr.ReadContentAsBoolean();
									break;

								case "id":
									id = tr.Value;
									break;
							}
						}

						Profile prf = new Profile();

						prf.Source = new SyncEndPoint(sourcePath);
						prf.Destination = new SyncEndPoint(destinationPath);
						prf.Rule = new SyncRule((SyncRule.RuleType) rule);
						prf.Name = name;
						prf.CopyOnce = copyOnce;
						if(id.Length != 0)
						{
							prf.ProfileID = new Guid(id);
						}

						bp.m_Profiles.Add(prf);
					}
				}
			}
			catch (Exception ex)
			{
				throw new ApplicationException("Unable to load the default profile!\nReason: " + ex);
			}
			finally
			{
				tr.Close();
			}

			return bp;
		}

		public void SaveToFile(string i_path)
		{
			if (!Directory.Exists(Path.GetDirectoryName(i_path)))
			{
				Directory.CreateDirectory(Path.GetDirectoryName(i_path));
			}

			XmlTextWriter tw = new XmlTextWriter(i_path,
			                                     UTF8Encoding.UTF8);
			tw.WriteStartDocument();
			tw.WriteStartElement("tracks");

			foreach (Profile p in m_Profiles)
			{
				tw.WriteStartElement("syncPoint");
				tw.WriteAttributeString("sourcePath", p.Source.SerializedData);
				tw.WriteAttributeString("destinationPath", p.Destination.SerializedData);
				tw.WriteAttributeString("rule", ((int) p.Rule.Rule).ToString());
				tw.WriteAttributeString("name", p.Name);
				tw.WriteAttributeString("copyOnce", p.CopyOnce ? "1" : "0");
				tw.WriteAttributeString("id", p.ProfileID.ToString());
				tw.WriteEndElement();
			}

			tw.WriteEndElement();
			tw.Close();
		}

		/// <summary>
		/// Executes the loaded Profiles
		/// </summary>
		public void ExecuteAllProfiles()
		{
			m_Profiles.ForEach(delegate(Profile i_Profile)
			                   	{
			                   		i_Profile.Run();
			                   	});
		}

		#region Async Profile Execution Support

		/// <summary>
		/// Executes the loaded Profiles asyncroneously
		/// </summary>
		public void ExecuteAllProfilesAsync()
		{
			ExecuteAllProfilesAsync(m_Profiles);
		}

		/// <summary>
		/// Cancels the currently executed batch profile
		/// </summary>
		public void CancelBatch()
		{
			lock (this)
			{
				m_ProfilesToExecute.ForEach(delegate(Profile p)
				                            	{
				                            		p.Cancel();
				                            	});
			}
		}

		/// <summary>
		/// True if there are profiles still running
		/// </summary>
		public bool IsRunning
		{
			get { return m_AsyncExecutionThread != null && m_AsyncExecutionThread.IsAlive; }
		}

		/// <summary>
		/// Executes the loaded Profiles
		/// </summary>
		public void ExecuteAllProfilesAsync(ProfileList i_Profiles)
		{
			m_ProfilesToExecute.Clear();
			m_ProfilesToExecute.AddRange(i_Profiles);

			//Refresh each profile
			m_ProfilesToExecute.ForEach(delegate(Profile p)
			                            	{
			                            		p.Refresh();
			                            	});

			ThreadStart ts = new ThreadStart(ExecuteCurrentProfilesAsync);
			m_AsyncExecutionThread = new Thread(ts);
			m_AsyncExecutionThread.Start();
		}

		private void ExecuteCurrentProfilesAsync()
		{
			m_ProfilesToExecute.ForEach(delegate(Profile i_Profile)
			                            	{
			                            		i_Profile.Run();
			                            	});
		}

		#endregion

		/// <summary>
		/// List of currently loaded profiles
		/// </summary>
		public ProfileList Profiles
		{
			get { return m_Profiles; }
		}

		public Profile GetProfileByName(string i_Name)
		{
			foreach (Profile p in m_Profiles)
			{
				if (String.Compare(p.Name, i_Name, true) == 0)
				{
					return p;
				}
			}
			
			return null;
		}
	}
}