using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace ivware.Mobile.RAPISync.Core
{
	public class ProfileHistory
	{
		public class HistoryEntry
		{
			private DateTime m_ItemDate;
			private string m_SourcePath;
			private EntryType m_ItemType;
			private bool m_Ignored = false;
			private bool m_Unchanged = false;

			public enum EntryType
			{
				Source = 0,
				Destination = 1
			}

			public HistoryEntry(string i_data)
			{
				if (i_data == null || i_data.IndexOf(",") < 1)
					throw new ArgumentException("Invalid data " + i_data + " provided to construct a history entry!");

				char[] dels = {','};
				string[] data = i_data.Split(dels);

				m_ItemDate = DateTime.Parse(data[0].Trim());
				m_SourcePath = data[1].Trim();

				if (data[2].Trim() == "S")
					m_ItemType = EntryType.Source;
				else
					m_ItemType = EntryType.Destination;

				if (data.Length > 3 && data[3].Trim() == "I")
					m_Ignored = true;

				if (data.Length > 3 && data[3].Trim() == "U")
					m_Unchanged = true;
			}

			/// <summary>
			/// Gets a value indicating whether this <see cref="T:HistoryEntry"/> was ignored.
			/// in the last run
			/// </summary>
			/// <value><c>true</c> if ignored; otherwise, <c>false</c>.</value>
			public bool Ignored
			{
				get { return m_Ignored; }
			}


			/// <summary>
			/// Gets a value indicating whether this <see cref="T:HistoryEntry"/> is unchanged.
			/// in the last run
			/// </summary>
			/// <value><c>true</c> if unchanged; otherwise, <c>false</c>.</value>
			public bool Unchanged
			{
				get { return m_Unchanged; }
			}

			public DateTime ItemDate
			{
				get { return m_ItemDate; }
			}

			public string SourcePath
			{
				get { return m_SourcePath; }
			}

			/// <summary>
			/// Gets the type of the item Source or Destination.
			/// </summary>
			/// <value>The type of the item.</value>
			public EntryType ItemType
			{
				get { return m_ItemType; }
			}

			/// <summary>
			/// Returns true if this history entry represents a specific diffgram item
			/// </summary>
			/// <param name="i_Item"></param>
			/// <returns></returns>
			public bool IsFor(ItemDiffGram i_Item)
			{
				if (i_Item.Action == ItemDiffGram.SyncAction.CopyToDestination &&
				    (ItemType == EntryType.Source || Ignored || Unchanged))
				{
					if (string.Compare(i_Item.Source.ItemPath, m_SourcePath, true) == 0 &&
					    Utilities.AreEqual(i_Item.Source.LastWriteTime, ItemDate))
						return true;
					else
						return false;
				}

				if (i_Item.Action == ItemDiffGram.SyncAction.CopyToSource &&
				    (ItemType == EntryType.Destination || Ignored || Unchanged))
				{
					if (string.Compare(i_Item.Destination.ItemPath, m_SourcePath, true) == 0 &&
					    Utilities.AreEqual(i_Item.Destination.LastWriteTime, ItemDate))
						return true;
					else
						return false;
				}

				return false;
			}

			public override string ToString()
			{
				return string.Format("{0}, {1}, {2}", ItemType, SourcePath, ItemDate);
			}
		}

		private Profile m_Owner;
		private List<HistoryEntry> m_History = new List<HistoryEntry>();

		public int NumItems
		{
			get { return m_History.Count; }
		}

		private string m_HistoryFileLocation =
			Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "BeyondSync");

		public ProfileHistory(Profile i_Owner)
		{
			m_Owner = i_Owner;
		}

		/// <summary>
		/// Gets or sets the location (directory) where profile history file should be created.
		/// </summary>
		/// <value>The history file location.</value>
		public string HistoryFileLocation
		{
			get { return m_HistoryFileLocation; }
			set { m_HistoryFileLocation = value; }
		}

		/// <summary>
		/// Loads the history entries
		/// </summary>
		/// <param name="i_Profile"></param>
		internal void LoadHistory()
		{
			string historyFile = GetHistoryFile();
			m_History.Clear();

			TextReader r;

			if (File.Exists(historyFile))
				r = new StreamReader(GetHistoryFile(), false);
			else
				return;

			string data;

			try
			{
				do
				{
					data = r.ReadLine();

					if (data != null && !data.StartsWith("###"))
					{
						try
						{
							HistoryEntry e = new HistoryEntry(data);
							m_History.Add(e);
						}
						catch
						{
						}
					}
				} while (data != null);
			}
			finally
			{
				r.Close();
			}
		}

		/// <summary>
		/// Applies the history filter.
		/// Ignores the items that exist in the history
		/// </summary>
		internal void ApplyHistoryFilter()
		{
			LoadHistory();

			if (m_History.Count > 0)
			{
				Trace.WriteLine("Applying history from " + GetHistoryFile());

				foreach (ItemDiffGram item in m_Owner.Diffgram.Items)
				{
					if (HasItem(item))
						item.Action = ItemDiffGram.SyncAction.Ignore;
				}
			}
		}

		/// <summary>
		/// Deletes the history file.
		/// </summary>
		internal void CleanHistory()
		{
			string historyFile = GetHistoryFile();

			if (File.Exists(historyFile))
				File.Delete(historyFile);
		}

		/// <summary>
		/// Saves all the sucessful entries to the profile history
		/// </summary>
		/// <param name="i_Profile"></param>
		internal void SaveHistory()
		{

			if (!Directory.Exists(HistoryFileLocation))
				Directory.CreateDirectory(HistoryFileLocation);
			
			TextWriter w = new StreamWriter(GetHistoryFile(), false);

			w.WriteLine(string.Format("### Profile name: {0},  Last Executed On: {1}", m_Owner.Name, DateTime.Now));

			try
			{
				foreach (ItemDiffGram d in m_Owner.Diffgram.Items)
				{
					if (d.Action == ItemDiffGram.SyncAction.CopyToDestination &&
					    d.Result.Status == OperationResult.ResultStatus.Success)
					{
						w.WriteLine(string.Format("{0}, {1}, S", d.Source.LastWriteTime, d.Source.ItemPath));
					}

					if (d.Action == ItemDiffGram.SyncAction.CopyToSource &&
					    d.Result.Status == OperationResult.ResultStatus.Success)
					{
						w.WriteLine(string.Format("{0}, {1}, D", d.Destination.LastWriteTime, d.Destination.ItemPath));
					}

					if (d.Action == ItemDiffGram.SyncAction.Ignore)
					{
						if (d.Source.Exists)
							w.WriteLine(string.Format("{0}, {1}, S, I", d.Source.LastWriteTime, d.Source.ItemPath));
						else
							w.WriteLine(string.Format("{0}, {1}, D, I", d.Destination.LastWriteTime, d.Destination.ItemPath));
					}

					if (d.Action == ItemDiffGram.SyncAction.Unchanged)
					{
						w.WriteLine(string.Format("{0}, {1}, S, U", d.Source.LastWriteTime, d.Source.ItemPath));
						w.WriteLine(string.Format("{0}, {1}, D, U", d.Destination.LastWriteTime, d.Destination.ItemPath));
					}
				}
			}
			finally
			{
				w.Close();
			}

			LoadHistory();
		}

		/// <summary>
		/// True if the history has an item for this diffgram
		/// </summary>
		/// <param name="i_Item"></param>
		/// <returns></returns>
		internal bool HasItem(ItemDiffGram i_Item)
		{
			foreach (HistoryEntry e in m_History)
			{
				if (e.IsFor(i_Item))
					return true;
			}

			return false;
		}

		/// <summary>
		/// Gets the path and name of the history file.
		/// </summary>
		/// <param name="i_Profile">The profile.</param>
		/// <returns></returns>
		private string GetHistoryFile()
		{
			if (!Directory.Exists(m_HistoryFileLocation))
				Directory.CreateDirectory(m_HistoryFileLocation);

			string file = Utilities.CleanFileName(m_Owner.Name);
			file += "_" + m_Owner.ProfileID.ToString("N") + ".history";
			return Path.Combine(m_HistoryFileLocation, file);
		}

		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();
			foreach (HistoryEntry he in m_History)
			{
				sb.AppendLine(he.ToString());
			}

			return sb.ToString();
		}
	}
}