using System;
using System.IO;
using System.Text;

namespace ivware.Mobile.RAPISync.Core
{
	public class StoreDiffGram
	{
		private IFileStore m_Source, m_Dest;
		private SyncRule m_Rule;
		private ItemDiffGramList m_ItemDiffGrams = new ItemDiffGramList();
		private Exception m_LastError = null;


		public StoreDiffGram()
		{
			Clear();
		}

		public StoreDiffGram(IFileStore i_Source, IFileStore i_Dest, SyncRule i_Rule)
		{
			if (!i_Source.Initialized)
				m_LastError = new ApplicationException("Unable to connect to the Source folder!");

			if (!i_Source.StoreRoot.Exists)
				m_LastError = new FileNotFoundException("Source path: " + i_Source.StoreRoot.ItemPath + " does not exist!");

			if (!i_Dest.Initialized)
				m_LastError = new ApplicationException("Unable to connect to the Destination folder!");

			m_Source = i_Source;
			m_Dest = i_Dest;
			m_Rule = i_Rule;

			if (!IsValid)
				return;

			try
			{
				GenerateFolderDiffgram(m_Source, m_Dest);
			}
			catch (Exception ex)
			{
				m_LastError = ex;
			}
		}

		private void GenerateFolderDiffgram(IFileStore i_src, IFileStore i_dest)
		{
			//Make sure we work on the latest data
			i_src.Refresh();
			i_dest.Refresh();

			//Find All in source that are the same or not in the destination
			foreach (StoreItem src in i_src.Items)
			{
				ItemDiffGram currenDg = null;

				foreach (StoreItem dst in i_dest.Items)
				{
					StoreItem.CompareResult res = src.CompareTo(dst);

					if (res != StoreItem.CompareResult.DifferentFiles)
					{
						currenDg = new ItemDiffGram(src, dst, m_Rule);
						break;
					}
				}

				//We did not find a matching item - create a placeholder
				if (currenDg == null)
				{
					StoreItem placeHolder = StoreItem.CreatePlaceholder(src, i_dest);
					currenDg = new ItemDiffGram(src, placeHolder, m_Rule);
				}

				m_ItemDiffGrams.Add(currenDg);
			}

			//Find all in the destination that are not in the source
			foreach (StoreItem dst in i_dest.Items)
			{
				bool found = false;

				foreach (StoreItem src in i_src.Items)
				{
					if (dst.CompareTo(src) != StoreItem.CompareResult.DifferentFiles)
					{
						found = true;
						break;
					}
				}

				if (!found)
				{
					StoreItem placeHolder = StoreItem.CreatePlaceholder(dst, i_src);
					m_ItemDiffGrams.Add(new ItemDiffGram(placeHolder, dst, m_Rule));
				}
			}
		}

		internal void Clear()
		{
			m_ItemDiffGrams.Clear();
			m_LastError = new ApplicationException("Store not initialized");
		}

		/// <summary>
		/// List of all items to be synchronized
		/// </summary>
		public ItemDiffGramList Items
		{
			get { return m_ItemDiffGrams; }
		}

		/// <summary>
		/// Finds all items based on the operation type
		/// </summary>
		/// <param name="i_Action"></param>
		/// <returns></returns>
		public ItemDiffGramList GetByAction(ItemDiffGram.SyncAction i_Action)
		{
			Predicate<ItemDiffGram> FindByAction = delegate(ItemDiffGram iItem)
			                                       	{
			                                       		return iItem.Action == i_Action;
			                                       	};

			ItemDiffGramList retVal = new ItemDiffGramList();

			retVal.AddRange(m_ItemDiffGrams.FindAll(FindByAction));
			return retVal;
		}

		/// <summary>
		/// Finds All items based on the Operation result
		/// </summary>
		/// <param name="i_Status"></param>
		/// <returns></returns>
		public ItemDiffGramList GetByResult(OperationResult.ResultStatus i_Status)
		{
			Predicate<ItemDiffGram> FindByResult = delegate(ItemDiffGram iItem)
			                                       	{
			                                       		return iItem.Result.Status == i_Status;
			                                       	};

			ItemDiffGramList retVal = new ItemDiffGramList();

			retVal.AddRange(m_ItemDiffGrams.FindAll(FindByResult));
			return retVal;
		}

		/// <summary>
		/// Finds All items based on the Operation result and Status
		/// </summary>
		/// <param name="i_Status"></param>
		/// <returns></returns>
		public ItemDiffGramList GetByActionAndResult(ItemDiffGram.SyncAction i_Action, OperationResult.ResultStatus i_Status)
		{
			Predicate<ItemDiffGram> FindByResultAndAction = delegate(ItemDiffGram iItem)
			                                                	{
			                                                		return
			                                                			(iItem.Result.Status == i_Status && iItem.Action == i_Action);
			                                                	};

			ItemDiffGramList retVal = new ItemDiffGramList();

			retVal.AddRange(m_ItemDiffGrams.FindAll(FindByResultAndAction));
			return retVal;
		}

		/// <summary>
		/// True if the Diffgram is valid
		/// </summary>
		public bool IsValid
		{
			get { return m_LastError == null; }
		}

		/// <summary>
		/// The exception that made the Diffgram invalid
		/// </summary>
		public Exception LastError
		{
			get { return m_LastError; }
		}

		public override string ToString()
		{
			if (!IsValid)
			{
				return "Invalid Diffgram! " + m_LastError.Message;
			}

			if (m_ItemDiffGrams.Count == 0)
			{
				return "Empty Diffgram!";
			}

			StringBuilder b = new StringBuilder();

			b.AppendLine("Store Diffgram\n--------------------------");

			foreach (ItemDiffGram i in m_ItemDiffGrams)
			{
				switch (i.Result.Status)
				{
					case OperationResult.ResultStatus.Canceled:
						b.Append("CNL ");
						break;

					case OperationResult.ResultStatus.Failed:
						b.Append("XXX ");
						break;

					case OperationResult.ResultStatus.Ignored:
						b.Append("IGN ");
						break;

					case OperationResult.ResultStatus.NotStarted:
						b.Append("... ");
						break;

					case OperationResult.ResultStatus.Success:
						b.Append("OK  ");
						break;
				}

				b.Append(i.Source);

				switch (i.Action)
				{
					case ItemDiffGram.SyncAction.CopyToDestination:
						b.Append(" ----> ");
						break;

					case ItemDiffGram.SyncAction.CopyToSource:
						b.Append(" <---- ");
						break;

					case ItemDiffGram.SyncAction.DeleteFromDestination:
						b.Append(" ----X ");
						break;

					case ItemDiffGram.SyncAction.Unchanged:
						b.Append(" ===== ");
						break;

					case ItemDiffGram.SyncAction.Ignore:
						b.Append(" =IGN= ");
						break;
				}

				b.Append(i.Destination);
				b.AppendLine();

				if (i.Result.Status == OperationResult.ResultStatus.Failed)
				{
					b.AppendFormat("---- Reason to Fail: {0}\n\n", i.Result.ReasonFailed);
				}
			}

			return b.ToString();
		}
	}
}