using System;
using System.IO;

namespace ivware.Mobile.RAPISync.Core
{
  public class StoreItem
  {
    protected string m_PlaceHolderPath;
    protected IFileStore m_ItemStore;
    bool m_IsDirectory = true;
    public static readonly char[] DIR_SEPS = { '/', '\\' };
    const short TIME_COMPARE_PRECISION = 2; //Number of seconds to ignore in the time differences

    public enum CompareResult
    {
      Same,
      OlderCopyOfTheSameFile,
      NewerCopyOfTheSameFile,
      DifferentFiles,
      Unknown
    };

    /// <summary>
    /// Constructor
    /// </summary>
    public StoreItem()
    {
    }

    /// <summary>
    /// The location of the item in the store
    /// </summary>
    public virtual string ItemPath
    {
      get
      {

        if (ItemStore == null)
          return m_PlaceHolderPath;

        return Path.Combine(ItemStore.StoreRoot.ItemPath, ItemRelativePath);
      }
    }

    /// <summary>
    /// The store that contains this item
    /// </summary>
    public virtual IFileStore ItemStore
    {
      get
      {
        return m_ItemStore;
      }
    }

    /// <summary>
    /// Compares 2 StoreItems
    /// </summary>
    /// <param name="i_rhs"></param>
    /// <returns></returns>
    public override bool Equals(object i_rhs)
    {
      if (i_rhs == null || !(i_rhs is StoreItem))
        return false;

      return CompareTo((StoreItem)i_rhs) == CompareResult.Same;
    }

    public virtual Object NativeItem
    {
      get
      {
        return null;
      }
    }

    /// <summary>
    /// Relative path to the root of the sync tree
    /// </summary>
    public virtual string ItemRelativePath
    {
      get
      {
        return m_PlaceHolderPath;
      }
    }

    public override int GetHashCode()
    {
      return ItemPath.GetHashCode();
    }

    /// <summary>
    /// True if the item represents a placeholder (virtual file)
    /// </summary>
    public virtual bool IsPlaceholder
    {
      get
      {
        return true;
      }
    }

    public virtual CompareResult CompareTo(StoreItem i_Rhs)
    {
      if (ItemRelativePath != i_Rhs.ItemRelativePath)
        return CompareResult.DifferentFiles;

      if (IsDirectory != i_Rhs.IsDirectory)
        return CompareResult.DifferentFiles;

      if (IsPlaceholder && i_Rhs.IsPlaceholder)
        return CompareResult.Same;

      if (i_Rhs.IsPlaceholder)
        return CompareResult.NewerCopyOfTheSameFile;

      else if (!i_Rhs.IsPlaceholder && IsPlaceholder)
        return CompareResult.OlderCopyOfTheSameFile;

      if (IsDirectory)
        return CompareResult.Same;

      //Check to see if the times differ within a precision
      TimeSpan diff = LastWriteTime - i_Rhs.LastWriteTime;

      if (Math.Abs(diff.TotalSeconds) < TIME_COMPARE_PRECISION)
        return CompareResult.Same;

      else if (diff.TotalSeconds < 0)
        return CompareResult.OlderCopyOfTheSameFile;

      else if (diff.TotalSeconds > 0)
        return CompareResult.NewerCopyOfTheSameFile;

      else
        return CompareResult.Unknown;
    }

    public override string ToString()
    {
      if (ItemRelativePath.Length == 0)
        return string.Format("store root ({0})", ItemPath);
      
      if (IsPlaceholder)
        return "[" + ItemRelativePath + "]";
      else
        return ItemRelativePath;
    }

    /// <summary>
    /// The time the file las tlast modified
    /// </summary>
    public virtual DateTime LastWriteTime
    {
      get
      {
        return DateTime.MinValue;
      }
    }

    /// <summary>
    /// The size of the file in bytes
    /// </summary>
    public virtual long Size
    {
      get
      {
        return 0;
      }
    }

    /// <summary>
    /// The size of the file as string
    /// </summary>
    public virtual string SizeAsString
    {
      get
      {
        if (Size < 1024)
					return string.Format("{0} Bytes", Size);
        else if (Size < 1000 * 1024)
          return string.Format("{0:F0} KB", (float)Size / (float)1024);
        else
          return string.Format("{0:F1} MB", (float)Size / (1000 * 1024));
      }
    }

    /// <summary>
    /// Creates a placeholder Item 
    /// (item that representa a missing file in this items's store)
    /// </summary>
    /// <param name="i_Rhs"></param>
    /// <returns></returns>
    public static StoreItem CreatePlaceholder(StoreItem i_Rhs, IFileStore i_Store)
    {
      StoreItem item = new StoreItem();
      item.m_PlaceHolderPath = i_Rhs.ItemRelativePath;
      item.m_ItemStore = i_Store;
      item.m_IsDirectory = i_Rhs.IsDirectory;
      return item;
    }

    /// <summary>
    /// Creates a placeholder Item 
    /// (item that representa a missing file in this items's store)
    /// </summary>
    /// <param name="i_Rhs"></param>
    /// <returns></returns>
    public static StoreItem CreatePlaceholder(string i_Rhs)
    {
      StoreItem item = new StoreItem();
      item.m_PlaceHolderPath = i_Rhs;
      return item;
    }

    /// <summary>
    /// True if the file is ReadOnly
    /// </summary>
    public virtual bool IsReadOnly
    {
      get
      {
        return false;
      }
    }

    /// <summary>
    /// Deletes the item from the store
    /// </summary>
    internal virtual void Delete()
    {
      ItemStore.DeleteItem(this);
    }

    /// <summary>
    /// Copies another over the current item
    /// </summary>
    /// <param name="i_storeItem"></param>
    internal virtual void CopyFrom(StoreItem i_storeItem)
    {
      ItemStore.CopyItem(i_storeItem, this);
    }

    /// <summary>
    /// True if the Item represents a directory
    /// </summary>
    public virtual bool IsDirectory
    {
      get
      {
        return m_IsDirectory;
      }
    }

    /// <summary>
    /// True if this Item physically exists in the store
    /// </summary>
    public virtual bool Exists
    {
      get
      {
        return false;
      }
    }
  }
}
