using System;
using System.IO;
using ivware.Mobile.RAPISync.Core;

namespace ivware.Mobile.RAPISync.Store
{
  /// <summary>
  /// Summary description for FileSystemStore.
  /// </summary>
  public class FileSystemStore : IFileStore
  {
    private StoreItem m_Root;
    private StoreItemList m_ItemList;
		private bool m_Initialized = false;

  	#region IFileStore Members

    /// <summary>
    /// Fired before Copy
    /// </summary>
    public event OnBeforeCopy BeforeCopy;

    /// <summary>
    /// Fired after copy
    /// </summary>
    public event OnAfterCopy AfterCopy;

    /// <summary>
    /// Fired before Delete
    /// </summary>
    public event OnBeforeDelete BeforeDelete;

    /// <summary>
    /// Fired after delete
    /// </summary>
    public event OnAfterDelete AfterDelete;

    public string Description
    {
      get { return "NTFS/FAT File system"; }
    }

    /// <summary>
    /// Supports free space information
    /// </summary>
    public bool CanGetFreeSpace
    {
      get { return true; }
    }

    /// <summary>
    /// Free space at the Store
    /// </summary>
    public long FreeSpace
    {
      get
      {
        if (m_Root is FileSystemStoreItem)
        {
          return ((FileSystemStoreItem) m_Root).Drive.AvailableFreeSpace;
        }

        return -1;
      }
    }

    public StoreItemList Items
    {
      get
      {
        if (m_ItemList == null)
        {
          m_ItemList = GetFolderStructure(m_Root);
          m_ItemList.Add(m_Root);
        }
        return m_ItemList;
      }
    }

    private StoreItemList GetFolderStructure(StoreItem i_Item)
    {
      StoreItemList list = new StoreItemList();

      if (i_Item.Exists)
      {
        FileSystemInfo item = (FileSystemInfo)(i_Item.NativeItem);

        if ((item.Attributes & FileAttributes.Directory) == 0)
          throw new ArgumentException("This item is not a directory!");

        // Process the directory list
        foreach (FileSystemInfo entry in ((DirectoryInfo)item).GetFileSystemInfos())
        {
          FileSystemStoreItem i = new FileSystemStoreItem(entry, this);
          list.Add(i);

          if (i.IsDirectory)
            list.AddRange(GetFolderStructure(i));
        }
      }
      return list;
    }

    public void CopyItem(StoreItem i_source, StoreItem i_destination)
    {

			Exception copyProblem = null;
    	
      if (i_source.IsDirectory != i_destination.IsDirectory)
        throw new ArgumentException(
          String.Format("Attempting to copy file into directory or vice versa!\n{0} -> {1}", i_source, i_destination));

      try
      {
        if (BeforeCopy != null && BeforeCopy(this, i_source, i_destination) != true)
          return;

        if (i_destination.IsPlaceholder)
          MakePathTo(i_destination);

        if (!i_destination.IsDirectory)
          File.Copy(i_source.ItemPath, i_destination.ItemPath,true);
      }
    	catch(Exception e)
    	{
				copyProblem = e;
    	}
      finally
      {
        if (AfterCopy != null)
					AfterCopy(this, i_source, i_destination, copyProblem);
      }

			if (copyProblem != null)
				throw copyProblem;
    }

    private void MakePathTo(StoreItem i_destination)
    {
      //Get the path part
      string path = i_destination.ItemPath;

      if (!i_destination.IsDirectory)
        path = Path.GetDirectoryName(path);

      if (!Directory.Exists(path))
        Directory.CreateDirectory(path);
    }

    public void DeleteItem(StoreItem i_destination)
    {
			Exception deleteException = null;
      FileSystemInfo item = ((FileSystemInfo) i_destination.NativeItem);

      item.Refresh();

      if (!item.Exists)
        return;

      try
      {
        if (BeforeDelete != null && BeforeDelete(this, i_destination) != true)
          return;

        if (i_destination.IsDirectory)
          ((DirectoryInfo) item).Delete(true);
        else
          item.Delete();
      }
      catch (UnauthorizedAccessException e)
      {
        if (i_destination.IsReadOnly)
          throw new UnauthorizedAccessException(e.Message + " The item is marked as READ ONLY!");
      }
    	catch(Exception ex)
    	{
				deleteException = ex;
    	}
      finally
      {
        if (AfterDelete != null)
          AfterDelete(this, i_destination, deleteException);
      }

			if (deleteException != null)
				throw deleteException;
    }

    public void InitializeRoot(StoreItem i_Root)
    {
			try
			{
				FileSystemStoreItem src = new FileSystemStoreItem(i_Root.ItemPath, this);

				FileSystemInfo item = (FileSystemInfo)(src.NativeItem);

				if ((item.Attributes & FileAttributes.Directory) == 0)
					throw new ArgumentException("Store Root must be a directory!");

				Refresh();
				m_Root = src;
				m_Initialized = true;
			}
    	catch
    	{
				m_Initialized = false;
    	}
    }

    public StoreItem StoreRoot
    {
      get { return m_Root; }
    }

    #endregion

    internal string CalculateRelativePath(FileSystemStoreItem i_Item)
    {
      string root = m_Root.ItemPath.ToLower().Replace(@"\", "/");
      string item = i_Item.ItemPath.ToLower().Replace(@"\", "/");

      if (item.IndexOf(root) != 0)
        return string.Empty;

      string path = i_Item.ItemPath.Substring(root.Length);
      char[] trims = {'/', '\\'};
      return path.Trim(trims);
    }

    /// <summary>
    /// Refreshes teh store
    /// </summary>
    public void Refresh()
    {
      m_ItemList = null;
    }

  	public bool Initialized
  	{
  		get { return m_Initialized; }
  	}

  	public string Protocol
  	{
			get
			{
				return string.Empty;
			}
  	}
  }
}