using System;
using System.IO;
using System.Threading;
using ivware.Mobile.RAPISync.Core;
using ivware.Mobile.RAPISync.Store;
using NUnit.Framework;

namespace ivware.Mobile.RAPISync.UnitTests
{
  [TestFixture]
  public class TestFileSystemStore
  
  {
    [TestFixtureSetUp]
    public void FixtureSetUp()
    {
      TestUtilities_FS.CleanupFolder(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder XX");
      TestUtilities_FS.CleanupFolder(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder X");
      TestUtilities_FS.CleanupFolder(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y");
      TestUtilities_FS.CleanupFolder(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1", "TestAuto*.*");
    }

    [SetUp]
    public void SetUp()
    {
    }

    [TearDown]
    public void TearDown()
    {
    }

    [TestFixtureTearDown]
    public void FixtureTearDown()
    {
    }


    [Test]
    public void TestSyncPoint()
    {
      SyncEndPoint s = new SyncEndPoint(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1");
      IFileStore store = s.GetStore();
      Assert.IsTrue(store is FileSystemStore);
    }

    [Test]
    public void TestFileStore()
    {
      FileSystemStoreItem i = new FileSystemStoreItem(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1");
      FileSystemStore store = new FileSystemStore();
      store.InitializeRoot(i);
      Assert.AreEqual(9, store.Items.Count);
      Assert.AreEqual("Folder 1-1", store.Items[0].ItemRelativePath);
    }


    [Test]
    public void TestFileSystemRunProfile()
    {
      SyncEndPoint src = new SyncEndPoint(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1", StoreFactory.Store.FileSystem);
      SyncEndPoint dst = new SyncEndPoint(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder XX");
      ProfileTester.TestRunProfile(src, dst);
    }


    [Test]
    public void TestBatchProfile()
    {
      string profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1\Folder 1-1,";
      profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y\Folder 1-1,";
      profile += " -o";
      profile += "\n";
      profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1\Folder 1-2, ";
      profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y\Folder 1-2, ";
      profile += " -o";

      BatchProfile bp = new BatchProfile(profile);
      Assert.AreEqual(2, bp.Profiles.Count);

      Assert.AreEqual(2, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(2, bp.Profiles[1].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(Profile.State.Completed, bp.Profiles[0].ProfileState);
      Assert.AreEqual(Profile.State.Completed, bp.Profiles[1].ProfileState);
      Assert.AreEqual(true, bp.Profiles[0].MadeChanges);
      Assert.AreEqual(true, bp.Profiles[1].MadeChanges);

      //Now delete
      profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder X,";
      profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y,";
      profile += " -o";

      bp = new BatchProfile(profile);
      Assert.AreEqual(1, bp.Profiles.Count);

      Assert.AreEqual(4, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(Profile.State.Completed, bp.Profiles[0].ProfileState);
      Assert.AreEqual(true, bp.Profiles[0].MadeChanges);
    	
    	string savePath = Path.Combine(Configuration.DefaultProfilePath, "TestSave.xml");
    	
    	bp.SaveToFile(savePath);

			BatchProfile bp1 = BatchProfile.LoadFromFile(savePath);

			Assert.AreEqual(bp.Profiles.Count, bp1.Profiles.Count);
			Assert.AreEqual(bp.Profiles[0].Name, bp1.Profiles[0].Name);
			Assert.AreEqual(bp.Profiles[0].Source.Root.ItemPath, bp1.Profiles[0].Source.Root.ItemPath);
			Assert.AreEqual(bp.Profiles[0].Destination.Root.ItemPath, bp1.Profiles[0].Destination.Root.ItemPath);
			Assert.AreEqual(bp.Profiles[0].Rule.Rule, bp1.Profiles[0].Rule.Rule);
			Assert.AreEqual(bp.Profiles[0].CopyOnce, bp1.Profiles[0].CopyOnce);
			Assert.AreEqual(bp.Profiles[0].ProfileID, bp1.Profiles[0].ProfileID);
    	
    	File.Delete(savePath);

    }


		[Test]
		public void TestBatchProfileAsync()
		{
			string profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1\Folder 1-1,";
			profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y\Folder 1-1,";
			profile += " -o";
			profile += "\n";
			profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1\Folder 1-2, ";
			profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y\Folder 1-2, ";
			profile += " -o";

			BatchProfile bp = new BatchProfile(profile);
			Assert.AreEqual(2, bp.Profiles.Count);

			Assert.AreEqual(2, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
			Assert.AreEqual(2, bp.Profiles[1].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);

			bp.ExecuteAllProfilesAsync();
			
			while(bp.IsRunning)
			{
				Thread.Sleep(200);
			}

			Assert.AreEqual(Profile.State.Completed, bp.Profiles[0].ProfileState);
			Assert.AreEqual(Profile.State.Completed, bp.Profiles[1].ProfileState);
			Assert.AreEqual(true, bp.Profiles[0].MadeChanges);
			Assert.AreEqual(true, bp.Profiles[1].MadeChanges);

			//Now delete
			profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder X,";
			profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y,";
			profile += " -o";

			bp = new BatchProfile(profile);
			Assert.AreEqual(1, bp.Profiles.Count);

			Assert.AreEqual(4, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

			bp.ExecuteAllProfilesAsync();

			while (bp.IsRunning)
			{
				Thread.Sleep(200);
			}

			Assert.AreEqual(Profile.State.Completed, bp.Profiles[0].ProfileState);
			Assert.AreEqual(true, bp.Profiles[0].MadeChanges);


		}


    [Test]
    public void TestFileCopyOnce()
    {
      string profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1,";
      profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y,";
      profile += " -o,";
      profile += " test,";
      profile += " -h";

      BatchProfile bp = new BatchProfile(profile);
      Assert.AreEqual(1, bp.Profiles.Count);
      Assert.IsTrue(bp.Profiles[0].CopyOnce);
      
      bp.Profiles[0].CleanHistory();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      
      bp.ExecuteAllProfiles();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);
      
      TestUtilities_FS.CleanupFolder(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y");

      bp.Profiles[0].Refresh();
      
      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(0, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(9, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);

      //Try one more time 
      bp.Profiles[0].Refresh();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(0, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(9, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);

      //Add a brand new file to the destination ans make sure it gets deleted
      TestUtilities_FS.CreateTestFileIn(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder Y");

      bp.Profiles[0].Refresh();

      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);
      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(9, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);

      //Add a brand new file to the source ans make sure it gets copied
      string file = TestUtilities_FS.CreateTestFileIn(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1");

      bp.Profiles[0].Refresh();

      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(9, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);

      bp.Profiles[0].Refresh();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(2, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(0, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(10, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);

      //Touch the file an dmake sure it gets copied
      File.SetLastWriteTime(file, DateTime.Now.AddSeconds(5));

      bp.Profiles[0].Refresh();

      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(9, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);
      
    }
    
    [Test]
    public void TestFileDiffGram()
    {
      FileSystemStoreItem src = new FileSystemStoreItem(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1");
      IFileStore src_store = StoreFactory.CreateStore(StoreFactory.Store.FileSystem);
      src_store.InitializeRoot(src);

      FileSystemStoreItem dest = new FileSystemStoreItem(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1");
      IFileStore dest_store = StoreFactory.CreateStore(StoreFactory.Store.FileSystem);
      dest_store.InitializeRoot(dest);

      SyncRule rule = new SyncRule(SyncRule.RuleType.SourceOverridesDestination);

      StoreDiffGram d = new StoreDiffGram(src_store, dest_store, rule);

      Assert.AreEqual(9, d.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.CopyToSource).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

      src = new FileSystemStoreItem(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder S");
      src_store = StoreFactory.CreateStore(StoreFactory.Store.FileSystem);
      src_store.InitializeRoot(src);

      dest = new FileSystemStoreItem(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder D");
      dest_store = StoreFactory.CreateStore(StoreFactory.Store.FileSystem);
      dest_store.InitializeRoot(dest);

      rule = new SyncRule(SyncRule.RuleType.SourceOverridesDestination);

      d = new StoreDiffGram(src_store, dest_store, rule);

      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      Assert.AreEqual(3, d.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.CopyToSource).Count);
      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

      rule.Rule = SyncRule.RuleType.SyncBothWays;

      d = new StoreDiffGram(src_store, dest_store, rule);

      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(3, d.GetByAction(ItemDiffGram.SyncAction.CopyToSource).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);
    }
  }
}